/* mutils.c
 * Wed Sep 28 17:41:45 PDT 2005
 * staka
 *
 * my utility functions
 */

#include "mutils.h"

void *
m_malloc(size_t size)
{

#ifdef CHECKMALLOC
  return check_malloc(size);
#else
  void *pt;

  if ((pt=malloc(size))==NULL) {
    perror("m_malloc");
    exit(1);
  }
  return pt;
#endif
}

void *
m_realloc(void *ptr, size_t size)
{
#ifdef CHECKMALLOC
  return check_realloc(ptr, size);
#else
  void *pt;

  if ((pt=realloc(ptr, size))==NULL) {
    perror("m_realloc");
    exit(1);
  }
  return pt;
#endif
}


void
m_free(void *ptr)
{
#ifdef CHECKMALLOC
  check_free(ptr);
  return;
#else 
  free(ptr);
#endif
}

int
isLittle(void)
{
  int n = 1;
  if (*(char*)&n) {
    return(1);
  } else {
    return(0);
  }
}

/* apply byte swap if this machine uses little endian */
int *
bswapli32(int *p)
{
   char temp,*q;
   
   if (isLittle()) {
     q = (char*) p;
     temp = *q; *q = *(q+3); *(q+3) = temp;
     temp = *(q+1); *(q+1) = *(q+2); *(q+2) = temp;
   }
   return p;
}

/* apply byte swap if this machine uses little endian */
float *
bswaplf32(float *p)
{
   char temp,*q;
   
   if (isLittle()) {
     q = (char*) p;
     temp = *q; *q = *(q+3); *(q+3) = temp;
     temp = *(q+1); *(q+1) = *(q+2); *(q+2) = temp;
   }
   return p;
}


/* 
   Do not use this as it cause numeric error for some values of float.
   i.e. if bswaplf(d) is not valid as float, then = operation does not
   keep the identity of the bit sequence.
float
bswaplf(float d)
{
  float s=d;
  char *ps=((char*)&s), *pd=((char*)&d);

  if (isLittle()) {
    pd[0] = ps[3];
    pd[1] = ps[2];
    pd[2] = ps[1];
    pd[3] = ps[0];
  }

  return d;
}
*/

char *
m_strdup(const char *s)
{
  char *cp;
  if ((cp=strdup(s))==NULL) {
    perror(s);
    exit(1);
  } else {
    return cp;
  }
}

char *
m_chomp(char *str)
{
  int len;
  
  len = strlen(str);
  if (len>0) {
    if (str[len-1] == '\n') {
      str[len-1] = '\0';
    }
  }
  return str;
}

FILE *
m_fopen(const char *path, const char *mode)
{
  FILE *fp;
  if ((fp=fopen(path, mode))==NULL) {
    perror(path);
    exit(1);
  }
  return fp;
}

int
m_fclose(FILE *stream)
{
  if (fclose(stream)!=0) {
    perror("fclose");
    exit(1);
  }
  return 0;
}

size_t
m_fread(void *ptr, size_t size, size_t nmemb, FILE *stream, char *fname)
{
  size_t retv;
  char buf[1000];

  retv = fread(ptr, size, nmemb, stream);
  if (retv == nmemb) {
    return(nmemb);
  } else {
    sprintf(buf, "Error read failed %s: retv %d size %d nmemb %d", fname, retv, size, nmemb);
    perror(buf);
    exit(1);
  }
}

void
m_fgets(char *s, int size, FILE *stream)
{
  if (fgets(s, size, stream) == NULL) {
    if (feof(stream)) {
      fprintf(stderr, "Can't read stream: EOF\n");
    } else {
      fprintf(stderr, "Can't read stream.\n");
      perror("m_fgets");
    }
    exit(1);
  }
}


/* File access control using fcntl. (Advisory file locking)
 * The file must be writable. I.e. it 
 * must be opend with r+, w, w+, a or a+)
 */
int
m_lock_fcntl(FILE *fp)
{
  struct flock lock;
  int fd;
  int i;
  char buf[100];

  lock.l_type = F_WRLCK;	/* Exclusive lock (This requires the file is writable) */
  lock.l_whence = SEEK_SET;
  lock.l_start = 0;
  lock.l_len = 0;
  lock.l_pid = getpid();

  fd = fileno(fp);

  for (i=0; i<5; i++) {
    if (fcntl(fd, F_SETLKW, &lock)) {
      if (i==0) {
	gethostname(buf, 100);
	buf[99] = '\0';
	fprintf(stderr, "%s\n", buf);
	fprintf(stderr, "WARN: fcntl failed to lock. Retry up to 5 times.\n");
      }
      perror("fcntl");
      if (i==4) {
	fprintf(stderr, "ERROR: fcntl failed 5 times.\n");
	exit(1);
      }
      sleep(1);
    } else {
      break;
    }
  }
  return fd;
}

/* Unlock the file.
 * Even without the call of this fuction, the file is 
 * automatically unlocked when it is closed or the process terminate.
 */
int
m_unlock_fcntl(FILE *fp)
{
  struct flock lock;
  int fd;
  int i;
  char buf[100];

  lock.l_type = F_WRLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start = 0;
  lock.l_len = 0;
  lock.l_pid = getpid();

  fd = fileno(fp);
  for (i=0; i<5; i++) {
    if (fcntl(fd, F_UNLCK, &lock)) {
      if (i==0) {
	gethostname(buf, 100);
	buf[99] = '\0';
	fprintf(stderr, "%s\n", buf);
	fprintf(stderr, "WARN: fcntl failed to unlock. Retry up to 5 times.\n");
      }
      perror("fcntl");
      if (i==4) {
	fprintf(stderr, "ERROR: fcntl failed 5 times.\n");
	exit(1);
      }
      sleep(1);
    } else {
      break;
    }
  }
  return fd;
}

int
m_lockdir(char *lockd)
{
  int w;

  w=0;
  while (1) {
    if (mkdir(lockd, S_IWUSR | S_IRUSR)) {
      if (errno != EEXIST) {
	perror("m_lock");
	exit(1);
      }
      w++;
      sleep(1);
    } else {
      break;
    }
  }
  return w;
}

int
m_unlockdir(char *lockd)
{

  if (rmdir(lockd)) {
    perror("m_unlock");
    exit(1);
  }
  return 0;
}
